@@ -35,16 +35,7 @@ reference or mutable reference, and has the advantage that now it can be a
 count bits using `.into()`.
 !*/
 
-use {
-	Bits,
-	Endian,
-	BigEndian,
-	BitVec,
-	TRUE,
-	FALSE,
-};
-use std::{
-	borrow::ToOwned,
+use core::{
 	cmp::{
 		Eq,
 		Ord,
@@ -57,12 +48,6 @@ use std::{
 		AsRef,
 		From,
 	},
-	fmt::{
-		self,
-		Debug,
-		Display,
-		Formatter,
-	},
 	hash::{
 		Hash,
 		Hasher,
@@ -90,6 +75,20 @@ use std::{
 	slice,
 };
 
+#[cfg(feature = "alloc")]
+use core::fmt::{
+	self,
+	Debug,
+	Display,
+	Formatter,
+};
+
+#[cfg(all(feature = "alloc", not(feature = "std")))]
+use alloc::borrow::ToOwned;
+
+#[cfg(feature = "std")]
+use std::borrow::ToOwned;
+
 /** A compact slice of bits, whose cursor and storage type can be customized.
 
 `BitSlice` is a newtype wrapper over `[T]`, and as such can only be held by
@@ -118,14 +117,14 @@ created by using the `From` implementation on `&BitSlice` and `&mut BitSlice`.
   downstream implementation, and can only be implemented in this crate.
 **/
 #[cfg_attr(nightly, repr(transparent))]
-pub struct BitSlice<E = BigEndian, T = u8>
-where E: Endian, T: Bits {
+pub struct BitSlice<E = crate::BigEndian, T = u8>
+where E: crate::Endian, T: crate::Bits {
 	_endian: PhantomData<E>,
 	inner: [T],
 }
 
 impl<E, T> BitSlice<E, T>
-where E: Endian, T: Bits {
+where E: crate::Endian, T: crate::Bits {
 	/// Gets the bit value at the given position.
 	///
 	/// The index value is a semantic count, not a bit address. It converts to a
@@ -134,10 +133,12 @@ where E: Endian, T: Bits {
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let bv = bitvec![0, 0, 1, 0, 0];
 	/// let bits: &BitSlice = &bv;
 	/// assert!(bits.get(2));
+	/// # }
 	/// ```
 	pub fn get(&self, index: usize) -> bool {
 		assert!(index < self.len(), "Index out of range!");
@@ -153,11 +154,13 @@ where E: Endian, T: Bits {
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let mut bv = bitvec![0; 5];
 	/// let bits: &mut BitSlice = &mut bv;
 	/// bits.set(2, true);
 	/// assert!(bits.get(2));
+	/// # }
 	/// ```
 	pub fn set(&mut self, index: usize, value: bool) {
 		assert!(index < self.len(), "Index out of range!");
@@ -179,6 +182,7 @@ where E: Endian, T: Bits {
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let all = bitvec![1; 10];
 	/// let any = bitvec![0, 0, 1, 0, 0];
@@ -189,6 +193,7 @@ where E: Endian, T: Bits {
 	/// assert!(!any.all());
 	/// assert!(!some.all());
 	/// assert!(!none.all());
+	/// # }
 	/// ```
 	pub fn all(&self) -> bool {
 		//  Gallop the filled elements
@@ -208,7 +213,7 @@ where E: Endian, T: Bits {
 				}
 			}
 		}
-		return true;
+		true
 	}
 
 	/// Returns true if *any* bit in the slice is set (logical `∨`).
@@ -225,6 +230,7 @@ where E: Endian, T: Bits {
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let all = bitvec![1; 10];
 	/// let any = bitvec![0, 0, 1, 0, 0];
@@ -235,6 +241,7 @@ where E: Endian, T: Bits {
 	/// assert!(any.any());
 	/// assert!(some.any());
 	/// assert!(!none.any());
+	/// # }
 	/// ```
 	pub fn any(&self) -> bool {
 		//  Gallop the filled elements
@@ -254,7 +261,7 @@ where E: Endian, T: Bits {
 				}
 			}
 		}
-		return false;
+		false
 	}
 
 	/// Returns true if *any* bit in the slice is unset (logical `¬∧`).
@@ -271,6 +278,7 @@ where E: Endian, T: Bits {
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let all = bitvec![1; 10];
 	/// let any = bitvec![0, 0, 1, 0, 0];
@@ -281,6 +289,7 @@ where E: Endian, T: Bits {
 	/// assert!(any.not_all());
 	/// assert!(some.not_all());
 	/// assert!(none.not_all());
+	/// # }
 	/// ```
 	pub fn not_all(&self) -> bool {
 		!self.all()
@@ -300,6 +309,7 @@ where E: Endian, T: Bits {
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let all = bitvec![1; 10];
 	/// let any = bitvec![0, 0, 1, 0, 0];
@@ -310,6 +320,7 @@ where E: Endian, T: Bits {
 	/// assert!(!any.not_any());
 	/// assert!(!some.not_any());
 	/// assert!(none.not_any());
+	/// # }
 	/// ```
 	pub fn not_any(&self) -> bool {
 		!self.any()
@@ -332,6 +343,7 @@ where E: Endian, T: Bits {
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let all = bitvec![1; 2];
 	/// let some = bitvec![1, 0];
@@ -340,6 +352,7 @@ where E: Endian, T: Bits {
 	/// assert!(!all.some());
 	/// assert!(some.some());
 	/// assert!(!none.some());
+	/// # }
 	/// ```
 	pub fn some(&self) -> bool {
 		self.any() && self.not_all()
@@ -350,9 +363,11 @@ where E: Endian, T: Bits {
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let bv = bitvec![1, 0, 1, 0, 1];
 	/// assert_eq!(bv.count_ones(), 3);
+	/// # }
 	/// ```
 	pub fn count_ones(&self) -> usize {
 		self.into_iter().filter(|b| *b).count()
@@ -363,9 +378,11 @@ where E: Endian, T: Bits {
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let bv = bitvec![0, 1, 0, 1, 0];
 	/// assert_eq!(bv.count_zeros(), 3);
+	/// # }
 	/// ```
 	pub fn count_zeros(&self) -> usize {
 		self.into_iter().filter(|b| !b).count()
@@ -376,10 +393,12 @@ where E: Endian, T: Bits {
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let bv = bitvec![1; 10];
 	/// let bits: &BitSlice = &bv;
 	/// assert_eq!(bits.len(), 10);
+	/// # }
 	/// ```
 	pub fn len(&self) -> usize {
 		self.inner.len()
@@ -394,17 +413,21 @@ where E: Endian, T: Bits {
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let bv = bitvec![1; 10];
 	/// let bits: &BitSlice = &bv;
 	/// assert_eq!(bits.elts(), 1);
+	/// # }
 	/// ```
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let bv = bitvec![1; 16];
 	/// let bits: &BitSlice = &bv;
 	/// assert_eq!(bits.elts(), 2);
+	/// # }
 	/// ```
 	pub fn elts(&self) -> usize {
 		self.len() >> T::BITS
@@ -419,17 +442,21 @@ where E: Endian, T: Bits {
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let bv = bitvec![1; 10];
 	/// let bits: &BitSlice = &bv;
 	/// assert_eq!(bits.bits(), 2);
+	/// # }
 	/// ```
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let bv = bitvec![1; 16];
 	/// let bits: &BitSlice = &bv;
 	/// assert_eq!(bits.bits(), 0);
+	/// # }
 	/// ```
 	pub fn bits(&self) -> u8 {
 		self.len() as u8 & T::MASK
@@ -440,17 +467,21 @@ where E: Endian, T: Bits {
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let bv = bitvec![];
 	/// let bits: &BitSlice = &bv;
 	/// assert!(bits.is_empty());
+	/// # }
 	/// ```
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let bv = bitvec![0; 5];
 	/// let bits: &BitSlice = &bv;
 	/// assert!(!bits.is_empty());
+	/// # }
 	/// ```
 	pub fn is_empty(&self) -> bool {
 		self.len() == 0
@@ -461,7 +492,7 @@ where E: Endian, T: Bits {
 	/// The iterator returned from this method implements `ExactSizeIterator`
 	/// and `DoubleEndedIterator` just as the consuming `.into_iter()` method’s
 	/// iterator does.
-	pub fn iter<'a>(&'a self) -> Iter<'a, E, T> {
+	pub fn iter(&self) -> Iter<E, T> {
 		self.into_iter()
 	}
 
@@ -475,6 +506,7 @@ where E: Endian, T: Bits {
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let mut bv = bitvec![1; 8];
 	/// let bref: &mut BitSlice = &mut bv;
@@ -487,8 +519,9 @@ where E: Endian, T: Bits {
 	///     }
 	/// });
 	/// assert_eq!(&[0b01010101], bref.as_ref());
+	/// # }
 	/// ```
-	pub fn for_each<'a, F>(&'a mut self, op: F)
+	pub fn for_each<F>(&mut self, op: F)
 	where F: Fn(usize, bool) -> bool {
 		for idx in 0 .. self.len() {
 			let tmp = self.get(idx);
@@ -512,17 +545,23 @@ where E: Endian, T: Bits {
 	/// # Examples
 	///
 	/// ```rust,ignore
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let bv = bitvec![1; 10];
 	/// let bits: &BitSlice = &bv;
 	/// assert_eq!(bits.elts(), 1);
 	/// assert_eq!(bits.raw_len(), 2);
+	/// # }
 	/// ```
+	///
+	/// This test is never compiled because the functions it calls are not
+	/// accessible to the test crate.
 	pub(crate) fn raw_len(&self) -> usize {
 		self.elts() + if self.bits() > 0 { 1 } else { 0 }
 	}
 
 	/// Prints a type header into the Formatter.
+	#[cfg(feature = "alloc")]
 	pub(crate) fn fmt_header(&self, fmt: &mut Formatter) -> fmt::Result {
 		write!(fmt, "BitSlice<{}, {}>", E::TY, T::TY)
 	}
@@ -531,13 +570,14 @@ where E: Endian, T: Bits {
 	///
 	/// The debug flag indicates whether to indent each line (`Debug` does,
 	/// `Display` does not).
+	#[cfg(feature = "alloc")]
 	pub(crate) fn fmt_body(&self, fmt: &mut Formatter, debug: bool) -> fmt::Result {
 		let (elts, bits) = T::split(self.len());
 		let len = self.raw_len();
 		let buf = self.as_ref();
 		let alt = fmt.alternate();
-		for idx in 0 .. elts {
-			Self::fmt_element(fmt, &buf[idx])?;
+		for (idx, elt) in buf.iter().take(elts).enumerate() {
+			Self::fmt_element(fmt, elt)?;
 			if idx < len - 1 {
 				match (alt, debug) {
 					// {}
@@ -558,13 +598,19 @@ where E: Endian, T: Bits {
 	}
 
 	/// Formats a whole storage element of the data slice.
+	#[cfg(feature = "alloc")]
 	pub(crate) fn fmt_element(fmt: &mut Formatter, elt: &T) -> fmt::Result {
 		Self::fmt_bits(fmt, elt, T::WIDTH)
 	}
 
 	/// Formats a partial element of the data slice.
+	#[cfg(feature = "alloc")]
 	pub(crate) fn fmt_bits(fmt: &mut Formatter, elt: &T, bits: u8) -> fmt::Result {
-		use std::fmt::Write;
+		use core::fmt::Write;
+
+		#[cfg(not(feature = "std"))]
+		use alloc::string::String;
+
 		let mut out = String::with_capacity(bits as usize);
 		for bit in 0 .. bits {
 			let cur = E::curr::<T>(bit);
@@ -575,20 +621,23 @@ where E: Endian, T: Bits {
 }
 
 /// Creates a new `BitVec` out of a `BitSlice`.
+#[cfg(feature = "alloc")]
 impl<E, T> ToOwned for BitSlice<E, T>
-where E: Endian, T: Bits {
-	type Owned = BitVec<E, T>;
+where E: crate::Endian, T: crate::Bits {
+	type Owned = crate::BitVec<E, T>;
 
 	/// Clones a borrowed `BitSlice` into an owned `BitVec`.
 	///
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let src = bitvec![0; 5];
 	/// let src_ref: &BitSlice = &src;
 	/// let dst = src_ref.to_owned();
 	/// assert_eq!(src, dst);
+	/// # }
 	/// ```
 	fn to_owned(&self) -> Self::Owned {
 		let mut out = Self::Owned::with_capacity(self.len());
@@ -604,10 +653,10 @@ where E: Endian, T: Bits {
 }
 
 impl<E, T> Eq for BitSlice<E, T>
-where E: Endian, T: Bits {}
+where E: crate::Endian, T: crate::Bits {}
 
 impl<E, T> Ord for BitSlice<E, T>
-where E: Endian, T: Bits {
+where E: crate::Endian, T: crate::Bits {
 	fn cmp(&self, rhs: &Self) -> Ordering {
 		match self.partial_cmp(rhs) {
 			Some(ord) => ord,
@@ -623,12 +672,13 @@ where E: Endian, T: Bits {
 /// The equality condition requires that they have the same number of total bits
 /// and that each pair of bits in semantic order are identical.
 impl<A, B, C, D> PartialEq<BitSlice<C, D>> for BitSlice<A, B>
-where A: Endian, B: Bits, C: Endian, D: Bits {
+where A: crate::Endian, B: crate::Bits, C: crate::Endian, D: crate::Bits {
 	/// Performs a comparison by `==`.
 	///
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let l: BitVec<LittleEndian, u16> = bitvec![LittleEndian, u16; 0, 1, 0, 1];
 	/// let r: BitVec<BigEndian, u32> = bitvec![BigEndian, u32; 0, 1, 0, 1];
@@ -636,6 +686,7 @@ where A: Endian, B: Bits, C: Endian, D: Bits {
 	/// let ls: &BitSlice<_, _> = &l;
 	/// let rs: &BitSlice<_, _> = &r;
 	/// assert!(ls == rs);
+	/// # }
 	/// ```
 	fn eq(&self, rhs: &BitSlice<C, D>) -> bool {
 		let (l, r) = (self.iter(), rhs.iter());
@@ -655,12 +706,13 @@ where A: Endian, B: Bits, C: Endian, D: Bits {
 /// If one of the slices is exhausted before they differ, the longer slice is
 /// greater.
 impl<A, B, C, D> PartialOrd<BitSlice<C, D>> for BitSlice<A, B>
-where A: Endian, B: Bits, C: Endian, D: Bits {
+where A: crate::Endian, B: crate::Bits, C: crate::Endian, D: crate::Bits {
 	/// Performs a comparison by `<` or `>`.
 	///
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let a = bitvec![0, 1, 0, 0];
 	/// let b = bitvec![0, 1, 0, 1];
@@ -670,6 +722,7 @@ where A: Endian, B: Bits, C: Endian, D: Bits {
 	/// let cref: &BitSlice = &c;
 	/// assert!(aref < bref);
 	/// assert!(bref < cref);
+	/// # }
 	/// ```
 	fn partial_cmp(&self, rhs: &BitSlice<C, D>) -> Option<Ordering> {
 		for (l, r) in self.iter().zip(rhs.iter()) {
@@ -686,18 +739,20 @@ where A: Endian, B: Bits, C: Endian, D: Bits {
 /// Gives write access to all elements in the underlying storage, including the
 /// partially-filled tail element (if present).
 impl<E, T> AsMut<[T]> for BitSlice<E, T>
-where E: Endian, T: Bits {
+where E: crate::Endian, T: crate::Bits {
 	/// Accesses the underlying store.
 	///
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let mut bv: BitVec = bitvec![0, 0, 0, 0, 0, 0, 0, 0, 1];
 	/// for elt in bv.as_mut() {
 	///   *elt += 2;
 	/// }
 	/// assert_eq!(&[2, 130], bv.as_ref());
+	/// # }
 	/// ```
 	fn as_mut(&mut self) -> &mut [T] {
 		let (ptr, len): (*mut T, usize) = (self.as_mut_ptr(), self.raw_len());
@@ -708,16 +763,18 @@ where E: Endian, T: Bits {
 /// Gives read access to all elements in the underlying storage, including the
 /// partially-filled tail element (if present).
 impl<E, T> AsRef<[T]> for BitSlice<E, T>
-where E: Endian, T: Bits {
+where E: crate::Endian, T: crate::Bits {
 	/// Accesses the underlying store.
 	///
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let bv = bitvec![0, 0, 0, 0, 0, 0, 0, 0, 1];
 	/// let bref: &BitSlice = &bv;
 	/// assert_eq!(&[0, 0b1000_0000], bref.as_ref());
+	/// # }
 	/// ```
 	fn as_ref(&self) -> &[T] {
 		let (ptr, len): (*const T, usize) = (self.as_ptr(), self.raw_len());
@@ -728,13 +785,14 @@ where E: Endian, T: Bits {
 /// Builds a `BitSlice` from a slice of elements. The resulting `BitSlice` will
 /// always completely fill the original slice, and will not have a partial tail.
 impl<'a, E, T> From<&'a [T]> for &'a BitSlice<E, T>
-where E: Endian, T: 'a + Bits {
+where E: crate::Endian, T: 'a + crate::Bits {
 	/// Wraps an `&[T: Bits]` in an `&BitSlice<E: Endian, T>`. The endianness
 	/// must be specified by the call site. The element type cannot be changed.
 	///
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let src = vec![1u8, 2, 3];
 	/// let borrow: &[u8] = &src;
@@ -746,6 +804,7 @@ where E: Endian, T: 'a + Bits {
 	/// assert!(bits.get(14)); // src[1] == 0b0000_0010
 	/// assert!(bits.get(22)); // src[2] == 0b0000_0011
 	/// assert!(bits.get(23));
+	/// # }
 	/// ```
 	fn from(src: &'a [T]) -> Self {
 		let (ptr, len): (*const T, usize) = (src.as_ptr(), src.len());
@@ -762,7 +821,7 @@ where E: Endian, T: 'a + Bits {
 /// `BitSlice` will always completely fill the original slice, and will not have
 /// a partial tail.
 impl<'a, E, T> From<&'a mut [T]> for &'a mut BitSlice<E, T>
-where E: Endian, T: 'a + Bits {
+where E: crate::Endian, T: 'a + crate::Bits {
 	/// Wraps an `&mut [T: Bits]` in an `&mut BitSlice<E: Endian, T>`. The
 	/// endianness must be specified by the call site. The element type cannot
 	/// be changed.
@@ -770,6 +829,7 @@ where E: Endian, T: 'a + Bits {
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let mut src = vec![1u8, 2, 3];
 	/// let borrow: &mut [u8] = &mut src;
@@ -778,6 +838,7 @@ where E: Endian, T: 'a + Bits {
 	/// assert!(bits.get(0));
 	/// bits.set(0, false);
 	/// assert!(!bits.get(0));
+	/// # }
 	/// ```
 	fn from(src: &'a mut [T]) -> Self {
 		let (ptr, len): (*mut T, usize) = (src.as_mut_ptr(), src.len());
@@ -800,13 +861,15 @@ where E: Endian, T: 'a + Bits {
 ///
 /// The alternate character `{:#?}` prints each element on its own line, rather
 /// than having all elements on the same line.
+#[cfg(feature = "alloc")]
 impl<E, T> Debug for BitSlice<E, T>
-where E: Endian, T: Bits {
+where E: crate::Endian, T: crate::Bits {
 	/// Renders the `BitSlice` type header and contents for debug.
 	///
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let bits: &BitSlice<LittleEndian, u16> = &bitvec![
 	///   LittleEndian, u16;
@@ -817,6 +880,7 @@ where E: Endian, T: Bits {
     ///     "BitSlice<LittleEndian, u16> [0101000011110101, 01]",
 	///     &format!("{:?}", bits)
 	/// );
+	/// # }
 	/// ```
 	fn fmt(&self, fmt: &mut Formatter) -> fmt::Result {
 		let alt = fmt.alternate();
@@ -839,16 +903,19 @@ where E: Endian, T: Bits {
 ///
 /// To see the in-memory representation, use `.as_ref()` to get access to the
 /// raw elements and print that slice instead.
+#[cfg(feature = "alloc")]
 impl<E, T> Display for BitSlice<E, T>
-where E: Endian, T: Bits {
+where E: crate::Endian, T: crate::Bits {
 	/// Renders the `BitSlice` contents for display.
 	///
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let bits: &BitSlice = &bitvec![0, 1, 0, 0, 1, 0, 1, 1, 0, 1];
 	/// assert_eq!("01001011 01", &format!("{}", bits));
+	/// # }
 	/// ```
 	fn fmt(&self, fmt: &mut Formatter) -> fmt::Result {
 		self.fmt_body(fmt, false)
@@ -857,7 +924,7 @@ where E: Endian, T: Bits {
 
 /// Writes the contents of the `BitSlice`, in semantic bit order, into a hasher.
 impl<E, T> Hash for BitSlice<E, T>
-where E: Endian, T: Bits {
+where E: crate::Endian, T: crate::Bits {
 	/// Writes each bit of the `BitSlice`, as a full `bool`, into the hasher.
 	fn hash<H>(&self, hasher: &mut H)
 	where H: Hasher {
@@ -873,7 +940,7 @@ where E: Endian, T: Bits {
 /// `ExactSizeIterator` as `BitSlice` has a known, fixed length, and
 /// `DoubleEndedIterator` as it has known ends.
 impl<'a, E, T> IntoIterator for &'a BitSlice<E, T>
-where E: Endian, T: 'a + Bits {
+where E: crate::Endian, T: 'a + crate::Bits {
 	type Item = bool;
 	type IntoIter = Iter<'a, E, T>;
 
@@ -882,6 +949,7 @@ where E: Endian, T: 'a + Bits {
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let bv = bitvec![1, 0, 1, 0, 1, 1, 0, 0];
 	/// let bref: &BitSlice = &bv;
@@ -890,6 +958,7 @@ where E: Endian, T: 'a + Bits {
 	///     if bit { count += 1; }
 	/// }
 	/// assert_eq!(count, 4);
+	/// # }
 	/// ```
 	fn into_iter(self) -> Self::IntoIter {
 		self.into()
@@ -912,7 +981,7 @@ where E: Endian, T: 'a + Bits {
 /// Subtraction can be implemented by negating the intended subtrahend yourself
 /// and then using addition, or by using `BitVec`s instead of `BitSlice`s.
 impl<'a, E, T> AddAssign<&'a BitSlice<E, T>> for BitSlice<E, T>
-where E: Endian, T: Bits {
+where E: crate::Endian, T: crate::Bits {
 	/// Performs unsigned wrapping addition in place.
 	///
 	/// # Examples
@@ -920,6 +989,7 @@ where E: Endian, T: Bits {
 	/// This example shows addition of a slice wrapping from MAX to zero.
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let nums: [BitVec; 3] = [
 	///     bitvec![1, 1, 1, 0],
@@ -933,9 +1003,10 @@ where E: Endian, T: Bits {
 	/// assert_eq!(numr, &nums[1] as &BitSlice);
 	/// *numr += &one;
 	/// assert_eq!(numr, &nums[2] as &BitSlice);
+	/// # }
 	/// ```
 	fn add_assign(&mut self, addend: &'a BitSlice<E, T>) {
-		use std::iter::repeat;
+		use core::iter::repeat;
 		//  zero-extend the addend if it’s shorter than self
 		let mut addend_iter = addend.into_iter().rev().chain(repeat(false));
 		let mut c = false;
@@ -957,20 +1028,22 @@ where E: Endian, T: Bits {
 /// the result into `self`. If the other bitstream ends before `self` does, it
 /// is extended with zero, clearing all remaining bits in `self`.
 impl<E, T, I> BitAndAssign<I> for BitSlice<E, T>
-where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
+where E: crate::Endian, T: crate::Bits, I: IntoIterator<Item=bool> {
 	/// `AND`s a bitstream into a slice.
 	///
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let lhs: &mut BitSlice = &mut bitvec![0, 1, 0, 1, 0, 1];
 	/// let rhs                =      bitvec![0, 0, 1, 1];
 	/// *lhs &= rhs;
 	/// assert_eq!("000100", &format!("{}", lhs));
+	/// # }
 	/// ```
 	fn bitand_assign(&mut self, rhs: I) {
-		use std::iter::repeat;
+		use core::iter::repeat;
 		for (idx, other) in (0 .. self.len()).zip(rhs.into_iter().chain(repeat(false))) {
 			let val = self.get(idx) & other;
 			self.set(idx, val);
@@ -982,17 +1055,19 @@ where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
 /// result into `self`. If the other bitstream ends before `self` does, it is
 /// extended with zero, leaving all remaining bits in `self` as they were.
 impl<E, T, I> BitOrAssign<I> for BitSlice<E, T>
-where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
+where E: crate::Endian, T: crate::Bits, I: IntoIterator<Item=bool> {
 	/// `OR`s a bitstream into a slice.
 	///
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let lhs: &mut BitSlice = &mut bitvec![0, 1, 0, 1, 0, 1];
 	/// let rhs                =      bitvec![0, 0, 1, 1];
 	/// *lhs |= rhs;
 	/// assert_eq!("011101", &format!("{}", lhs));
+	/// # }
 	/// ```
 	fn bitor_assign(&mut self, rhs: I) {
 		for (idx, other) in (0 .. self.len()).zip(rhs.into_iter()) {
@@ -1006,20 +1081,22 @@ where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
 /// the result into `self`. If the other bitstream ends before `self` does, it
 /// is extended with zero, leaving all remaining bits in `self` as they were.
 impl<E, T, I> BitXorAssign<I> for BitSlice<E, T>
-where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
+where E: crate::Endian, T: crate::Bits, I: IntoIterator<Item=bool> {
 	/// `XOR`s a bitstream into a slice.
 	///
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let lhs: &mut BitSlice = &mut bitvec![0, 1, 0, 1, 0, 1];
 	/// let rhs                =      bitvec![0, 0, 1, 1];
 	/// *lhs ^= rhs;
 	/// assert_eq!("011001", &format!("{}", lhs));
+	/// # }
 	/// ```
 	fn bitxor_assign(&mut self, rhs: I) {
-		use std::iter::repeat;
+		use core::iter::repeat;
 		for (idx, other) in (0 .. self.len()).zip(rhs.into_iter().chain(repeat(false))) {
 			let val = self.get(idx) ^ other;
 			self.set(idx, val);
@@ -1030,7 +1107,7 @@ where E: Endian, T: Bits, I: IntoIterator<Item=bool> {
 /// Indexes a single bit by semantic count. The index must be less than the
 /// length of the `BitSlice`.
 impl<'a, E, T> Index<usize> for &'a BitSlice<E, T>
-where E: Endian, T: 'a + Bits {
+where E: crate::Endian, T: 'a + crate::Bits {
 	type Output = bool;
 
 	/// Looks up a single bit by semantic count.
@@ -1038,17 +1115,16 @@ where E: Endian, T: 'a + Bits {
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let bv = bitvec![0, 0, 1, 0, 0];
 	/// let bits: &BitSlice = &bv;
 	/// assert!(bits[2]);
 	/// assert!(!bits[3]);
+	/// # }
 	/// ```
 	fn index(&self, index: usize) -> &Self::Output {
-		match self.get(index) {
-			true => &TRUE,
-			false => &FALSE,
-		}
+		if self.get(index) { &true} else { &false }
 	}
 }
 
@@ -1058,7 +1134,7 @@ where E: Endian, T: 'a + Bits {
 ///
 /// This index is not recommended for public use.
 impl<'a, E, T> Index<(usize, u8)> for &'a BitSlice<E, T>
-where E: Endian, T: 'a + Bits {
+where E: crate::Endian, T: 'a + crate::Bits {
 	type Output = bool;
 
 	/// Looks up a single bit by storage element and bit indices. The bit index
@@ -1067,18 +1143,17 @@ where E: Endian, T: 'a + Bits {
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let mut bv = bitvec![0; 10];
 	/// bv.push(true);
 	/// let bits: &BitSlice = &bv;
 	/// assert!(bits[(1, 2)]); // 10
 	/// assert!(!bits[(1, 1)]); // 9
+	/// # }
 	/// ```
 	fn index(&self, (elt, bit): (usize, u8)) -> &Self::Output {
-		match self.get(T::join(elt, bit)) {
-			true => &TRUE,
-			false => &FALSE,
-		}
+		if self.get(T::join(elt, bit)) { &true } else { &false }
 	}
 }
 
@@ -1103,7 +1178,7 @@ where E: Endian, T: 'a + Bits {
 ///
 /// Because `BitSlice` cannot move, the negation is performed in place.
 impl<'a, E, T> Neg for &'a mut BitSlice<E, T>
-where E: Endian, T: 'a + Bits {
+where E: crate::Endian, T: 'a + crate::Bits {
 	type Output = Self;
 
 	/// Perform 2’s-complement fixed-width negation.
@@ -1118,26 +1193,31 @@ where E: Endian, T: 'a + Bits {
 	/// Negate an arbitrary positive number (first bit unset).
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let mut num = bitvec![0, 1, 1, 0];
 	/// - (&mut num as &mut BitSlice);
 	/// assert_eq!(num, bitvec![1, 0, 1, 0]);
+	/// # }
 	/// ```
 	///
 	/// Negate an arbitrary negative number. This example will use the above
 	/// result to demonstrate round-trip correctness.
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let mut num = bitvec![1, 0, 1, 0];
 	/// - (&mut num as &mut BitSlice);
 	/// assert_eq!(num, bitvec![0, 1, 1, 0]);
+	/// # }
 	/// ```
 	///
 	/// Negate the most negative number, which will become zero, and show
 	/// convergence at zero.
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let zero = bitvec![0; 10];
 	/// let mut num = bitvec![1; 10];
@@ -1145,6 +1225,7 @@ where E: Endian, T: 'a + Bits {
 	/// assert_eq!(num, zero);
 	/// - (&mut num as &mut BitSlice);
 	/// assert_eq!(num, zero);
+	/// # }
 	/// ```
 	fn neg(self) -> Self::Output {
 		if self.is_empty() || self.not_any() {
@@ -1174,7 +1255,7 @@ where E: Endian, T: 'a + Bits {
 /// `BitSlice` purview. `^=` also has the advantage of being a borrowing
 /// operator rather than a consuming/returning operator.
 impl<'a, E, T> Not for &'a mut BitSlice<E, T>
-where E: Endian, T: 'a + Bits {
+where E: crate::Endian, T: 'a + crate::Bits {
 	type Output = Self;
 
 	/// Inverts all bits in the slice.
@@ -1182,6 +1263,7 @@ where E: Endian, T: 'a + Bits {
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let mut bv = bitvec![0; 10];
 	/// let bits: &mut BitSlice = &mut bv;
@@ -1190,6 +1272,7 @@ where E: Endian, T: 'a + Bits {
 	/// //  is returned.
 	/// // assert_eq!(bits.as_ref(), &[!0, !0]);
 	/// assert_eq!(new_bits.as_ref(), &[!0, !0]);
+	/// # }
 	/// ```
 	fn not(self) -> Self::Output {
 		for elt in self.as_mut() {
@@ -1231,18 +1314,20 @@ __bitslice_shift!(u8, u16, u32, u64, i8, i16, i32, i64);
 ///
 /// A shift amount of zero is a no-op, and returns immediately.
 impl<E, T> ShlAssign<usize> for BitSlice<E, T>
-where E: Endian, T: Bits {
+where E: crate::Endian, T: crate::Bits {
 	/// Shifts a slice left, in place.
 	///
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let mut bv = bitvec![1, 1, 1, 0, 0, 0, 0, 0, 1];
 	/// let bits: &mut BitSlice = &mut bv;
 	/// *bits <<= 3;
 	/// assert_eq!("00000100 0", &format!("{}", bits));
 	/// //               ^ former tail
+	/// # }
 	/// ```
 	fn shl_assign(&mut self, shamt: usize) {
 		let len = self.len();
@@ -1326,18 +1411,20 @@ where E: Endian, T: Bits {
 ///
 /// A shift amount of zero is a no-op, and returns immediately.
 impl<E, T> ShrAssign<usize> for BitSlice<E, T>
-where E: Endian, T: Bits {
+where E: crate::Endian, T: crate::Bits {
 	/// Shifts a slice right, in place.
 	///
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let mut bv = bitvec![1, 0, 0, 0, 0, 0, 1, 1, 1];
 	/// let bits: &mut BitSlice = &mut bv;
 	/// *bits >>= 3;
 	/// assert_eq!("00010000 0", &format!("{}", bits));
 	/// //             ^ former head
+	/// # }
 	/// ```
 	fn shr_assign(&mut self, shamt: usize) {
 		let len = self.len();
@@ -1385,14 +1472,14 @@ where E: Endian, T: Bits {
 /// Permits iteration over a `BitSlice`
 #[doc(hidden)]
 pub struct Iter<'a, E, T>
-where E: 'a + Endian, T: 'a + Bits {
+where E: 'a + crate::Endian, T: 'a + crate::Bits {
 	inner: &'a BitSlice<E, T>,
 	head: usize,
 	tail: usize,
 }
 
 impl<'a, E, T> Iter<'a, E, T>
-where E: 'a + Endian, T: 'a + Bits {
+where E: 'a + crate::Endian, T: 'a + crate::Bits {
 	fn reset(&mut self) {
 		self.head = 0;
 		self.tail = self.inner.len();
@@ -1400,7 +1487,7 @@ where E: 'a + Endian, T: 'a + Bits {
 }
 
 impl<'a, E, T> DoubleEndedIterator for Iter<'a, E, T>
-where E: 'a + Endian, T: 'a + Bits {
+where E: 'a + crate::Endian, T: 'a + crate::Bits {
 	fn next_back(&mut self) -> Option<Self::Item> {
 		if self.tail > self.head {
 			self.tail -= 1;
@@ -1414,14 +1501,14 @@ where E: 'a + Endian, T: 'a + Bits {
 }
 
 impl<'a, E, T> ExactSizeIterator for Iter<'a, E, T>
-where E: 'a + Endian, T: 'a + Bits {
+where E: 'a + crate::Endian, T: 'a + crate::Bits {
 	fn len(&self) -> usize {
 		self.tail - self.head
 	}
 }
 
 impl<'a, E, T> From<&'a BitSlice<E, T>> for Iter<'a, E, T>
-where E: 'a + Endian, T: 'a + Bits {
+where E: 'a + crate::Endian, T: 'a + crate::Bits {
 	fn from(src: &'a BitSlice<E, T>) -> Self {
 		let len = src.len();
 		Self {
@@ -1433,7 +1520,7 @@ where E: 'a + Endian, T: 'a + Bits {
 }
 
 impl<'a, E, T> Iterator for Iter<'a, E, T>
-where E: 'a + Endian, T: 'a + Bits {
+where E: 'a + crate::Endian, T: 'a + crate::Bits {
 	type Item = bool;
 
 	fn next(&mut self) -> Option<Self::Item> {
@@ -1458,9 +1545,11 @@ where E: 'a + Endian, T: 'a + Bits {
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let bv = bitvec![BigEndian, u8; 0, 1, 0, 1, 0];
 	/// assert_eq!(bv.iter().count(), 5);
+	/// # }
 	/// ```
 	fn count(self) -> usize {
 		ExactSizeIterator::len(&self)
@@ -1474,11 +1563,13 @@ where E: 'a + Endian, T: 'a + Bits {
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let bv = bitvec![BigEndian, u8; 0, 0, 0, 1];
 	/// let mut bv_iter = bv.iter();
 	/// assert_eq!(bv_iter.len(), 4);
 	/// assert!(bv_iter.nth(3).unwrap());
+	/// # }
 	/// ```
 	///
 	/// This example intentionally overshoots the iterator bounds, which causes
@@ -1489,12 +1580,14 @@ where E: 'a + Endian, T: 'a + Bits {
 	/// before yielding the next.
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let bv = bitvec![BigEndian, u8; 0, 0, 0, 1];
 	/// let mut bv_iter = bv.iter();
 	/// assert!(bv_iter.nth(4).is_none());
 	/// assert!(!bv_iter.nth(2).unwrap());
 	/// assert!(bv_iter.nth(0).unwrap());
+	/// # }
 	/// ```
 	fn nth(&mut self, n: usize) -> Option<bool> {
 		self.head = self.head.saturating_add(n);
@@ -1506,17 +1599,21 @@ where E: 'a + Endian, T: 'a + Bits {
 	/// # Examples
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let bv = bitvec![BigEndian, u8; 0, 0, 0, 1];
 	/// assert!(bv.into_iter().last().unwrap());
+	/// # }
 	/// ```
 	///
 	/// Empty iterators return `None`
 	///
 	/// ```rust
+	/// # #[cfg(feature = "alloc")] {
 	/// use bitvec::*;
 	/// let bv = bitvec![];
 	/// assert!(bv.into_iter().last().is_none());
+	/// # }
 	/// ```
 	fn last(mut self) -> Option<bool> {
 		self.next_back()
